
use std::{path::Path, str::FromStr, iter::zip};

use itertools::Itertools;

use crate::{error::AdventError, input::read_into};

struct Record {
    time: u64,
    distance: u64,
}

impl FromStr for Record {
    type Err = AdventError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let mut lines = s.lines();
        let time = lines
            .nth(0).ok_or(AdventError("A zero'th row doesn't exist".into()))?
            .trim()
            .split_ascii_whitespace()
            .skip(1)
            .fold("".to_string(), |mut sb, s| {
                sb.push_str(s);
                sb
            }).parse::<u64>()?;
        
        let distance = lines
            .nth(0).ok_or(AdventError("A zero'th row doesn't exist".into()))?
            .trim()
            .split_ascii_whitespace()
            .skip(1)
            .fold("".to_string(), |mut sb, s| {
                sb.push_str(s); 
                sb
            }).parse::<u64>()?;

        Ok(Record{time, distance})
    }
}

struct RecordsOverview {
    records: Vec<Record>
}

impl FromStr for RecordsOverview {
    type Err = AdventError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let mut lines = s.lines();
        let times = lines
            .nth(0).ok_or(AdventError("A zero'th row doesn't exist".into()))?
            .trim()
            .split_ascii_whitespace()
            .skip(1)
            .map(|s| -> Result<u64, AdventError> {
                Ok(s.parse::<u64>()?)
            })
            .collect::<Result<Vec<u64>, AdventError>>()?;
        
        let distances = lines
            .nth(0).ok_or(AdventError("A zero'th row doesn't exist".into()))?
            .trim()
            .split_ascii_whitespace()
            .skip(1)
            .map(|s| -> Result<u64, AdventError> {
                Ok(s.parse::<u64>()?)
            })
            .collect::<Result<Vec<u64>, AdventError>>()?;
        
        let records: Vec<Record> = zip(times, distances).map(|(time, distance)| {
            Record{time, distance}
        }).collect_vec();
        
        Ok(RecordsOverview {
            records
        })
    }
}

pub fn number_of_record_beating_chances(max_time: i128, record: i128) -> u32 {
    let sqrt = ((max_time.pow(2) - 4 * record) as f64).sqrt();

    let answer1 = -((-max_time as f64) + sqrt) / 2f64;
    let answer2 = -((-max_time as f64) - sqrt) / 2f64;

    (answer2.ceil() - answer1.ceil()) as u32
}

pub fn one() -> Result<u64, AdventError> {
    let overview = read_into::<RecordsOverview>(Path::new("resources/input6.txt"))?;
    Ok(overview.records.iter().fold(1, |acc, record| {
        acc * number_of_record_beating_chances(record.time.into(), record.distance.into())
    }) as u64)
}

pub fn two() -> Result<u64, AdventError> {
    let record = read_into::<Record>(Path::new("resources/input6.txt"))?;
    Ok(number_of_record_beating_chances(record.time.into(), record.distance.into()) as u64) 
}